# frozen_string_literal: true

require "core/global"

module Core
  module Local
    # [public] Simple key/value store, localized to a thread or fiber.
    #
    class Store
      include Is::Global

      extend Forwardable
      def_delegators :state, :[], :[]=, :fetch, :delete, :clear, :count

      def initialize
        @key = :"__corerb_localized_state_#{object_id}"
      end

      # [public] Returns a proc that, when called, deletes the key from the global store.
      #
      def cleanup(key)
        proc { delete(key) }
      end

      # [public] Returns all the state for the current thread or fiber.
      #
      private def state
        Thread.current[@key] ||= {}
      end
    end
  end
end
